#ifndef MAZERUNNER_HH
#define MAZERUNNER_HH

//----------------------------------------------------------------------------
// pacman-like game
// copyright 2010 christopher rasmussen
// university of delaware
// v. 1.07
// * added score/time elapsed drawing
// * crossover collisions now detected
// * added wind terrain
// * added blue_ghost_wait_counter
// * starting to move functions, variables over to GameState class; making it const in call to update()
// * added "-blue" option for blue ghosts/no blue ghosts; saving .lay; new quit key
// * moved to MazeRunner.hh
// * added arena_mode flag, , ArenaAgent.hh include

//----------------------------------------------------------------------------

#include "BlockMaze.hh"
#include "Utils.hh"
#include "Agent.hh"
#include "GhostAgent.hh"
#include "RunnerAgent.hh"
#include "ArenaAgent.hh"
#include "TeamXAgent.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#define DO_SPLASH

const char QUIT_KEY           = '/';
const char SAVE_KEY           = '.';
const char NEXT_RUNNER_KEY    = 'm';

const int CONTINUE_PLAYING    = 0;
const int RUNNER_WIN          = 1;
const int GHOST_WIN           = 2;

// ARENA

const int EVEN_WIN            = 3;
const int ODD_WIN             = 4;
const int EVEN_ODD_TIE        = 5;

const int MAX_ARENA_UPDATES   = 250;

const int DEFAULT_ROWS        = 10;
const int DEFAULT_COLS        = 10;

const int NO_BLUE_GHOST       = 0;
const int STANDARD_BLUE_GHOST = 15;

#define ODD_COLOR     YELLOW_COLOR
#define EVEN_COLOR    GREEN_COLOR

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

class MazeRunner
{
public:

  MazeRunner(int rows, int cols, int, int);
  MazeRunner(string filename, int, int, bool);

  int main_loop(bool);   // run a full game
  void reset();          // go back to pre-game state

  bool do_step;                 // wait for keypress to advance game?

private:

  // general functions

  int update();
  void draw_splash();
  void draw_gameover(int);
  void draw();
  void draw_power_pills();
  bool handle_key();
  void compute_dimensions_lay(ifstream &, int &, int &);
  void load_lay(string, bool);
  void save_lay(string);
  bool check_for_power_pill_eaten(int &);

  // ARENA functions

  int arena_update();
  bool arena_check_for_power_pill_eaten(int &);
  int num_odd_pills(vector <Point_2D> &);
  int num_even_pills(vector <Point_2D> &);

  // variables

  int num_updates;              // how many global updates()'s have been called

  BlockMaze *M;                 // the layout of the environment
  vector <Agent *> agent;
  vector <Point_2D> power_pill, initial_power_pill;
  GameState GS;

  int runner_agent_index;       // where is runner in agent array
  int blue_ghost_time_limit;    // run with "-blue" option to change

  bool arena_mode;              // are we playing head-to-head (instead of pac-man style)?
  int arena_border_col;         // which column marks the boundary between the odd and even regions?
  int max_turns;                // when to stop even if game's not over yet

};

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#endif
